@@ -65,10 +65,11 @@ data structures
 import os
 import sys
 import yaml
+import pprint
 
 class Yaco(dict):
     """
-    Very loosely based on http://code.activestate.com/recipes/473786/ (r1)
+    Rather loosely based on http://code.activestate.com/recipes/473786/ (r1)
 
     >>> v= Yaco()
     >>> v.a = 1
@@ -101,7 +102,7 @@ class Yaco(dict):
         """
         return str(self.get_data())
 
-    def __setitem__(self, key, value):
+    def __setattr__(self, key, value):
         """
         Set the value of a key
         
@@ -141,6 +142,8 @@ class Yaco(dict):
             #setting a dict
             if isinstance(old_value, Yaco):
                 old_value.update(value)
+            elif isinstance(value, Yaco):
+                super(Yaco, self).__setitem__(key, value)
             else:
                 super(Yaco, self).__setitem__(key, Yaco(value))
         elif isinstance(value, list):
@@ -154,13 +157,16 @@ class Yaco(dict):
         rv = super(Yaco, self).has_key(key)
         return rv
 
-    def __getitem__(self, key):
+    def __getattr__(self, key):
         """
         >>> v= Yaco()
         >>> v.a = 18       
         >>> assert(v.a == 18)
         >>> assert(isinstance(v.a, int))
         """
+        if key == 'getVersion':
+            print self
+            print self.__dict__
         try:
             return super(Yaco, self).__getitem__(key)
         except KeyError:
@@ -195,8 +201,9 @@ class Yaco(dict):
         """
         if not data: return 
         for key, value in data.items():
-            if isinstance(value, Yaco):
-                raise Exception("Wow - updating with a Yaco - should not happen (%s = %s)!" % (key, value))
+            #if isinstance(value, Yaco):
+            #    raise Exception("Wow - updating with a Yaco - "+
+            #                    "should not happen (%s = %s)!" % (key, value))
 
             old_value = super(Yaco, self).get(key, None)
             if isinstance(value, dict):
@@ -211,8 +218,8 @@ class Yaco(dict):
             else:
                 super(Yaco, self).__setitem__(key, value)
 
-    __getattr__ = __getitem__
-    __setattr__ = __setitem__
+    __getitem__ = __getattr__
+    __setitem__ = __setattr__
 
     def copy(self):
         ch = Yaco(self)
@@ -231,17 +238,44 @@ class Yaco(dict):
         >>> y.load(tf.name)
         >>> assert(y.a[3][3].d == 4)
         """
+        from_file = os.path.abspath(os.path.expanduser(from_file))
         with open(from_file) as F:
             data = yaml.load(F)
         self.update(data)
 
+    def pretty(self):
+        """
+        Return data as a pprint.pformatted string
+        """
+        return pprint.pformat(self.get_data())
+    
     def get_data(self):
         """
-        Prepare & parse data for export             
+        Prepare & parse data for export
+
+        >>> y = Yaco()
+        >>> y.a = 1
+        >>> y.b = 2
+        >>> y._c = 3
+        >>> assert(y._c == 3)
+        >>> d = y.get_data()
+        >>> assert(d.has_key('a') == True)
+        >>> assert(d.has_key('b') == True)
+        >>> assert(d.has_key('_c') == False)
+        >>> y._private = ['b']
+        >>> d = y.get_data()
+        >>> assert(d.has_key('a') == True)
+        >>> assert(d.has_key('b') == False)
+        >>> assert(d.has_key('_c') == False)
+        
         """
         data = {}
+        _priv = self.get('_private', [])
         for k in self.keys():
-            if k[0] == '_': continue
+            if k in _priv:
+                continue
+            if k[0] == '_':
+                continue
             val = self[k]
             if isinstance(val, Yaco):
                 val = val.get_data()
